/*!
   Copyright 2019 Ron Buckton

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
/**
 * Indicates an object that has resources that can be explicitly disposed.
 */
export interface Disposable {
    /**
     * Dispose this object's resources.
     */
    [Disposable.dispose](): void;
}
export declare namespace Disposable {
    /**
     * A well-known symbol used to define an explicit resource disposal method on an object.
     */
    const dispose: unique symbol;
    /**
     * Determines whether a value is Disposable.
     * @deprecated Use `Disposable.hasInstance` instead.
     */
    function isDisposable(value: unknown): value is Disposable;
    /**
     * Creates a `Disposable` wrapper around a callback used to dispose of a resource.
     */
    function create(dispose: () => void): Disposable;
    /**
     * Creates a `Disposable` wrapper around a set of other disposables.
     * @param disposables An `Iterable` of `Disposable` objects.
     */
    function from(disposables: Iterable<Disposable | null | undefined>): Disposable;
    /**
     * Executes a callback with the provided `Disposable` resource, disposing the resource when the callback completes.
     */
    function use<T extends Disposable | null | undefined, U>(resource: T, callback: (resource: T) => U): U;
    const name = "Disposable";
    /**
     * Determines whether a value is Disposable.
     */
    function hasInstance(value: unknown): value is Disposable;
}
/**
 * Indicates an object that has resources that can be explicitly disposed asynchronously.
 */
export interface AsyncDisposable {
    /**
     * Dispose this object's resources.
     */
    [AsyncDisposable.asyncDispose](): Promise<void>;
}
export declare namespace AsyncDisposable {
    /**
     * A well-known symbol used to define an async explicit resource disposal method on an object.
     */
    const asyncDispose: unique symbol;
    /**
     * Determines whether a value is [[AsyncDisposable]].
     * @deprecated Use `AsyncDisposable.hasInstance` instead.
     */
    function isAsyncDisposable(value: unknown): value is AsyncDisposable;
    /**
     * Creates an `AsyncDisposable` wrapper around a callback used to dispose resources.
     */
    function create(dispose: () => void | PromiseLike<void>): AsyncDisposable;
    /**
     * Creates an `AsyncDisposable` wrapper around a set of other disposables.
     * @param resources An `Iterable` of `AsyncDisposable` or `Disposable` objects.
     */
    function from(resources: Iterable<AsyncDisposable | Disposable | null | undefined>): AsyncDisposable;
    /**
     * Executes a callback with the provided `AsyncDisposable` resource, disposing the resource when the callback completes asynchronously.
     */
    function use<T extends AsyncDisposable | Disposable | null | undefined, U>(resource: T, callback: (resource: T) => U | PromiseLike<U>): Promise<U>;
    const name = "AsyncDisposable";
    /**
     * Determines whether a value is [[AsyncDisposable]].
     */
    function hasInstance(value: unknown): value is AsyncDisposable;
}
//# sourceMappingURL=index.d.ts.map