"use strict";
/*!
   Copyright 2019 Ron Buckton

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
Object.defineProperty(exports, "__esModule", { value: true });
const internal_deprecate_1 = require("@esfx/internal-deprecate");
const internal_guards_1 = require("@esfx/internal-guards");
const internal_tag_1 = require("@esfx/internal-tag");
var Disposable;
(function (Disposable) {
    // #region Disposable
    /**
     * A well-known symbol used to define an explicit resource disposal method on an object.
     */
    Disposable.dispose = Symbol.for("@esfx/disposable:Disposable.dispose");
    // #endregion Disposable
    /**
     * Determines whether a value is Disposable.
     * @deprecated Use `Disposable.hasInstance` instead.
     */
    function isDisposable(value) {
        return Disposable.hasInstance(value);
    }
    Disposable.isDisposable = isDisposable;
    const disposablePrototype = internal_tag_1.defineTag({}, "Disposable");
    /**
     * Creates a `Disposable` wrapper around a callback used to dispose of a resource.
     */
    function create(dispose) {
        return Object.setPrototypeOf({
            [Disposable.dispose]() {
                dispose();
            }
        }, disposablePrototype);
    }
    Disposable.create = create;
    /**
     * Creates a `Disposable` wrapper around a set of other disposables.
     * @param disposables An `Iterable` of `Disposable` objects.
     */
    function from(disposables) {
        const disposablesArray = [];
        for (const disposable of disposables) {
            if (internal_guards_1.isMissing(disposable))
                continue;
            if (!isDisposable(disposable))
                throw new TypeError("Disposable element expected: disposables");
            disposablesArray.push(disposable);
        }
        return create(() => {
            for (const disposable of disposablesArray) {
                disposable[Disposable.dispose]();
            }
        });
    }
    Disposable.from = from;
    /**
     * Executes a callback with the provided `Disposable` resource, disposing the resource when the callback completes.
     */
    function use(resource, callback) {
        if (!internal_guards_1.isMissing(resource) && !isDisposable(resource))
            throw new TypeError("Disposable expected: resource");
        try {
            return callback(resource);
        }
        finally {
            if (!internal_guards_1.isMissing(resource)) {
                resource[Disposable.dispose]();
            }
        }
    }
    Disposable.use = use;
    Disposable.name = "Disposable";
    /**
     * Determines whether a value is Disposable.
     */
    function hasInstance(value) {
        return internal_guards_1.isObject(value)
            && Disposable.dispose in value;
    }
    Disposable.hasInstance = hasInstance;
})(Disposable = exports.Disposable || (exports.Disposable = {}));
var AsyncDisposable;
(function (AsyncDisposable) {
    /**
     * A well-known symbol used to define an async explicit resource disposal method on an object.
     */
    AsyncDisposable.asyncDispose = Symbol.for("@esfx/disposable:AsyncDisposable.asyncDispose");
    /**
     * Determines whether a value is [[AsyncDisposable]].
     * @deprecated Use `AsyncDisposable.hasInstance` instead.
     */
    function isAsyncDisposable(value) {
        return AsyncDisposable.hasInstance(value);
    }
    AsyncDisposable.isAsyncDisposable = isAsyncDisposable;
    const asyncDisposablePrototype = internal_tag_1.defineTag({}, "AsyncDisposable");
    /**
     * Creates an `AsyncDisposable` wrapper around a callback used to dispose resources.
     */
    function create(dispose) {
        return Object.setPrototypeOf({
            async [AsyncDisposable.asyncDispose]() {
                await dispose();
            }
        }, asyncDisposablePrototype);
    }
    AsyncDisposable.create = create;
    function asyncFromSyncDisposable(disposable) {
        return create(() => disposable[Disposable.dispose]());
    }
    function toAsyncDisposable(resource) {
        return AsyncDisposable.hasInstance(resource) ? resource :
            Disposable.hasInstance(resource) ? asyncFromSyncDisposable(resource) :
                undefined;
    }
    /**
     * Creates an `AsyncDisposable` wrapper around a set of other disposables.
     * @param resources An `Iterable` of `AsyncDisposable` or `Disposable` objects.
     */
    function from(resources) {
        const disposablesArray = [];
        for (const resource of resources) {
            if (!internal_guards_1.isMissing(resource) && !AsyncDisposable.hasInstance(resource) && !Disposable.hasInstance(resource)) {
                throw new TypeError("AsyncDisposable element expected: resources");
            }
            if (internal_guards_1.isMissing(resource))
                continue;
            disposablesArray.push(toAsyncDisposable(resource));
        }
        return create(async () => {
            for (const disposable of disposablesArray) {
                await disposable[AsyncDisposable.asyncDispose]();
            }
        });
    }
    AsyncDisposable.from = from;
    /**
     * Executes a callback with the provided `AsyncDisposable` resource, disposing the resource when the callback completes asynchronously.
     */
    async function use(resource, callback) {
        if (!internal_guards_1.isMissing(resource) && !Disposable.hasInstance(resource) && !AsyncDisposable.hasInstance(resource)) {
            throw new TypeError("AsyncDisposable expected: resource");
        }
        const disposable = toAsyncDisposable(resource);
        try {
            return await callback(resource);
        }
        finally {
            if (!internal_guards_1.isMissing(disposable)) {
                await disposable[AsyncDisposable.asyncDispose]();
            }
        }
    }
    AsyncDisposable.use = use;
    AsyncDisposable.name = "AsyncDisposable";
    /**
     * Determines whether a value is [[AsyncDisposable]].
     */
    function hasInstance(value) {
        return internal_guards_1.isObject(value)
            && AsyncDisposable.asyncDispose in value;
    }
    AsyncDisposable.hasInstance = hasInstance;
})(AsyncDisposable = exports.AsyncDisposable || (exports.AsyncDisposable = {}));
internal_deprecate_1.deprecateProperty(Disposable, "isDisposable", "Use 'Disposable.hasInstance' instead.");
internal_deprecate_1.deprecateProperty(AsyncDisposable, "isAsyncDisposable", "Use 'AsyncDisposable.hasInstance' instead.");
//# sourceMappingURL=index.js.map